package hero.session;

/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along wit this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: AdminSessionBean.java,v 1.6 2004/11/29 12:28:48 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.Constants;
import hero.util.EventConstants;
import hero.util.HeroException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;


/**
*
* The Admin Session Bean, is an stateful session bean that provides the admin API to get information about
* projects and activities in which the user is admin.<br><br>
* <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
* <br>
* First of all you have to import the Admin Session files:<br>
* <br>
* import hero.interfaces.AdminSessionLocalHome;<br>
* import hero.interfaces.AdminSessionLocal;<br>
* import hero.interfaces.AdminSessionHome;<br>
* import hero.interfaces.AdminSession;<br>
* import hero.interfaces.AdminSessionUtil;<br>
* <br>
* Now, it is time to create the Admin Session instance:<br>
* <br>
* Like this if you want to use local interfaces:<br><br>
* AdminSessionLocalHome adminh = (AdminSessionLocalHome)hero.interfaces.AdminSessionUtil.getLocalHome();<br>
* AdminSessionLocal adminsession = adminh.create();<br>
* <br>
* or like this if you use remote interfaces:<br>  
* <br>
* AdminSessionHome adminh = (AdminSessionHome)hero.interfaces.AdminSessionUtil.getHome();<br>
* AdminSession adminsession = adminh.create();<br>
* <br>
* <br>
* Now you can call all Admin Sessions methods...  
*
 *
 * @ejb:bean name="AdminSession"
 *           display-name="AdminSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/AdminSession"
 *           local-jndi-name="ejb/hero/AdminSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user,SuperAdmin" 
 * @jonas.bean
 *      ejb-name="AdminSession"
 *      jndi-name="ejb/hero/AdminSession"
 * 
 *
 **/

public class AdminSessionBean implements SessionBean, EventConstants  {


   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
   // Utility variable
	private static final Logger trace = Logger.getLogger(AdminSessionBean.class);
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
  
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
	* Get admin workflow models (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
    *
   **/        
	   public Collection getModels() throws HeroException {
	   trace.info("start by " + mContext.getCallerPrincipal().getName());
	    Collection result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();			
			Collection models =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.MODEL);
			Iterator mdls = models.iterator(); 
			while (mdls.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)mdls.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
   /**
	* Get instances list for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
    *
   **/        
	   public Collection getInstances() throws HeroException {
	   trace.info("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator inst = instances.iterator(); 
			while (inst.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)inst.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
   /**
	* Get activities for instances/cooperatives projects in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
   **/
	   public Collection getActivities() throws HeroException {
	   trace.info("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection activities = nodehome.findAdminActivities(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			Iterator act = activities.iterator();
			while (act.hasNext())
			{
				BnNodeLocal nd = (BnNodeLocal)act.next();
				result.add(nd.getBnNodeValue());				
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	  }	   	
	   
   /**
   * Create the AdminSession Bean
   *
   * @throws CreateException 
   *
   * @ejb.permission unchecked="yes" 
   * @ejb:create-method view-type="both"
   **/

    public void ejbCreate() throws CreateException {
	trace.debug("ejbCreate");
    }
    
    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }
    
    public void ejbRemove() {
    }
    
    public void ejbActivate() {
    }
    
    public void ejbPassivate() {
    }
    
}
